using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using ILOG.Diagrammer;
using System.Drawing;
using ILOG.Diagrammer.Windows.Forms;
using System.Drawing.Drawing2D;
using ILOG.Diagrammer.Graphic;

namespace ProcessView
{
        public class CircleInteractor : ViewInteractor
        {

            ILOG.Diagrammer.Point2D _startPt = Point2D.Empty;
            float _radius;
            ILOG.Diagrammer.Stroke _stroke = new Stroke(Color.Black, 1, DashStyle.Dot);

            public CircleInteractor()
                : this(MouseButtons.Left)
            {
            }

            public CircleInteractor(MouseButtons button)
                : base(button)
            {
            }

            public Stroke GhostStroke
            {
                get
                {
                    return _stroke;
                }
                set
                {
                    _stroke = value;
                }
            }

            protected override void OnMouseDown(MouseEventArgs e)
            {
                if (e.Button == Buttons)
                {
                    _startPt = new Point2D(e.X, e.Y);
                    StartInteraction();
                }
                base.OnMouseDown(e);
            }

            protected override void OnMouseMove(MouseEventArgs e)
            {
                if (InOperation)
                {
                    Rectangle bounds = GetGhostBounds();
                    Point2D currentPt = new Point2D(e.X, e.Y);
                    _radius = GetDistance(_startPt, currentPt);
                    if (bounds.IsEmpty)
                        bounds = GetGhostBounds();
                    else
                        bounds = Rectangle.Union(bounds, GetGhostBounds());
                    InvalidateView(bounds);
                }
                base.OnMouseMove(e);
            }

            protected override void OnMouseUp(MouseEventArgs e)
            {
                if (InOperation)
                {
                    if (_radius > 3)
                        DoIt();
                    Rectangle bounds = GetGhostBounds();
                    StopInteraction(true);
                    InvalidateView(bounds);
                }
                base.OnMouseUp(e);
            }

            protected override void OnKeyDown(KeyEventArgs e)
            {
                if (e.KeyCode == Keys.Escape)
                {
                    Rectangle bounds = GetGhostBounds();
                    View.Invalidate(bounds);
                    StopInteraction(false);
                }
                base.OnKeyDown(e);
            }

            protected System.Drawing.Rectangle GetGhostGeometry()
            {
                Rectangle bounds =
                  new Rectangle((int)_startPt.X - (int)Math.Floor(_radius),
                                (int)_startPt.Y - (int)Math.Floor(_radius),
                                (int)(2 * _radius + 1),
                                (int)(2 * _radius + 1));
                return bounds;
            }

            protected Rectangle GetGhostBounds()
            {
                Rectangle bounds = GetGhostGeometry();
                using (Pen pen = _stroke.GetPenForBounds(View.Transform))
                {
                    float width = pen.Width;
                    bounds.Inflate((int)width, (int)width);
                    bounds.Offset((int)(-width / 2), (int)(-width / 2));
                }
                return bounds;
            }

            protected override void DrawGhost(ILOG.Diagrammer.DrawingContext context)
            {
                if (InOperation)
                {
                    Graphics g = context.Graphics;
                    Rectangle bounds = GetGhostGeometry();
                    using (Pen pen = _stroke.GetPen(
                       new Rectangle2D(bounds.X, bounds.Y,
                                       bounds.Width, bounds.Height),
                       View.Transform))
                    {
                        g.DrawEllipse(pen, bounds);
                    }
                }
                else
                {
                    base.DrawGhost(context);
                }
            }

            protected override void StopInteraction(bool validate)
            {
                _startPt = Point2D.Empty;
                _radius = 0;
                base.StopInteraction(validate);
            }

            protected virtual void DoIt()
            {
                Rectangle rect = GetGhostGeometry();
                Ellipse ellipse = new Ellipse();
                Transform t = View.Content.Transform;
                t = t * View.Transform;
                t = t.Inverse();

                // Computes the points of the rect in the container coordinates
                Point2D[] points = GetPointsFromRectangle(rect);
                t.TransformPoints(points);
                // Sets the object's size
                if (rect.Width != 0f || rect.Height != 0f)
                {
                    float w = GetDistance(points[0], points[1]);
                    float h = GetDistance(points[1], points[2]);
                    ellipse.Size = new Size2D(w, h);
                }
                // Sets the object's location
                Point2D center = t.TransformPoint(_startPt);
                ellipse.Move(center, ContentAlignment.MiddleCenter);
                GraphicObject[] obj = new GraphicObject[] { ellipse };
                IManageChildren container = View.Content as IManageChildren;
                if (container != null)
                    container.AddChildren(obj);
                if (SelectOnCreate)
                {
                    View.Selection.SetSelectedObjects(obj);
                }
            }

            static private float GetDistance(Point2D p1, Point2D p2)
            {
                double a = (p2.X - p1.X);
                double b = (p2.Y - p1.Y);
                return (float)Math.Sqrt(a * a + b * b);
            }

            static public Point2D[] GetPointsFromRectangle(Rectangle2D rect)
            {
                return new Point2D[] {
      new Point2D(rect.Left, rect.Top),
      new Point2D(rect.Right, rect.Top),
      new Point2D(rect.Right, rect.Bottom),
      new Point2D(rect.Left, rect.Bottom) };
            }

        }
}
