using System;
using ILOG.Diagrammer;
using ILOG.Diagrammer.Graphic;
using System.Drawing;
using System.Windows.Forms;
using System.Collections;

namespace ProcessView
{
    public delegate void DrillDownHandler(object sender, DrillDownEventArgs args);

    public partial class ProcessDiagram : Group
    {
        private Rect _selectionRect;
        private GraphicObject _currentSelectedGO;
        private ArrayList _taskShapes = new ArrayList();

        /// <summary>
        /// Should only be used from within ProcessCanvas to stay consistent!
        /// A C++ friend class would be a solution here... J.
        /// </summary>
        private bool _readOnly = false;
        protected internal bool ReadOnly
        {
            get
            {
                return _readOnly;
            }
            set
            {
                _readOnly = value;
            }
        }

        
        public event DrillDownHandler DrillDownEvent;

        private void OnDrillDown(GraphicObject go)
        {
            if(DrillDownEvent != null)
            {
                DrillDownEvent(this, new DrillDownEventArgs(go));
            }
        }

        public ProcessDiagram()
        {
            InitializeComponent();	// required by the designer
            
            this._taskShapes.AddRange(new GraphicObject[4]{this.manualTask1, this.manualTask2, this.manualTask3, this.manualTask4});
            this.ChildrenHierarchyChanged += new ChildrenHierarchyChangeEventHandler(ProcessDiagram_ChildrenHierarchyChanged);
        }

        private void ProcessDiagram_ChildrenHierarchyChanged(object sender, ChildrenHierarchyChangeEventArgs e)
        {
            if((e == null) || (e.Children == null)) return;

            if(e.Action == ChildrenHierarchyChangeAction.Add)
            {
                foreach(GraphicObject go in e.Children)
                    if(go is SystemTaskUserSymbol)
                        this._taskShapes.Add(go);
            }
            else if(e.Action == ChildrenHierarchyChangeAction.Remove)
            {
                foreach(GraphicObject go in e.Children)
                    if(this._taskShapes.Contains(go))
                        this._taskShapes.Remove(go);
            }
        }

        void Task_BoundsChanged(object sender, EventArgs e)
        {
            GraphicObject go = (GraphicObject)sender;
            if(IsSelected(go))
            {
                AddSelection(go);
            }
        }

        public void Select(GraphicObject go)
        {
            if(_currentSelectedGO == go)
            {
                RemoveSelection();
                _currentSelectedGO = null;
            }
            else
            {
                this.AddSelection(go);
                _currentSelectedGO = go;
            }
        }

        private bool IsSelected(GraphicObject go)
        {
            if(go == _currentSelectedGO)
            {
                return true;
            }
            return false;
        }

        public void AddSelection(GraphicObject symbol2select)
        {
            this.RemoveSelection();

            if(symbol2select == null) return;

            _currentSelectedGO = symbol2select;

            Rectangle2D geometry = symbol2select.Bounds;

            // New Width & Left Position
            float newWidth = geometry.Width * (float)(1.15);
            float widthDelta = newWidth - geometry.Width;
            float offsetX = (widthDelta / 2) * -1;
            geometry.Width = newWidth;

            // New Height & Top Position
            float newHeight = geometry.Height * (float)(1.25);
            float heightDelta = newHeight - geometry.Height;
            float offsetY = (heightDelta / 2) * -1;
            geometry.Height = newHeight;

            geometry.Offset(offsetX, offsetY);

            //Create selection rect and add to diagram
            ILOG.Diagrammer.Graphic.Rect selectionRect = new ILOG.Diagrammer.Graphic.Rect(geometry);
            selectionRect.Fill = new SolidFill(Color.Red);
            selectionRect.Stroke = new Stroke(Color.Red);
            selectionRect.Opacity = (float)0.5;
            selectionRect.Stroke.Width = 4;
            selectionRect.Visibility = ILOG.Diagrammer.Visibility.Visible;
            selectionRect.Radius = new Size2D(10f, 10f);

            _selectionRect = selectionRect;

            symbol2select.BoundsChanged += new EventHandler(Task_BoundsChanged);

            this.Objects.Add(selectionRect);

            this.SetChildZOrder(selectionRect, this.GetChildZOrder(symbol2select) - 1);
        }

        public void RemoveSelection()
        {
            if(this._selectionRect != null)
            {
                if(_currentSelectedGO != null)
                {
                    this._currentSelectedGO.BoundsChanged -= new EventHandler(Task_BoundsChanged);
                    this._currentSelectedGO = null;
                }
                this.Objects.Remove(this._selectionRect);
            }
        }

        /*
        public bool HasDrillDownSelection
        {
            get
            {
                bool hasDrill = false;
                if(_currentSelectedGO != null)
                {
                    hasDrill = true;
                }
                return hasDrill;
            }
        }
         */

        public bool IsDiagramItem(GraphicObject go)
        {
            bool isDiagram = false;
            if(this._taskShapes.Contains(go))
                return true;

            return isDiagram;
        }

        public void Delete(GraphicObject[] graphicObjects)
        {
            foreach(GraphicObject go in graphicObjects)
            {
                if(this.IsSelected(go))
                {
                    this.RemoveSelection();
                    break;
                }
            }

            this.RemoveChildren(graphicObjects);
        }
    }

    public class DrillDownEventArgs:EventArgs
    {
        GraphicObject _go = null;
        public DrillDownEventArgs(GraphicObject go)
        {
            _go = go;
        }

        public GraphicObject GObject
        {
            get
            {
                return _go;
            }
        }

        public string TaskName
        {
            get
            {
                return _go.Name;
            }
        }
    }
}
