using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using ILOG.Diagrammer;
using ILOG.Diagrammer.Graphic;
using ILOG.Diagrammer.Windows.Forms;

namespace ProcessView
{
    public partial class WorkflowCanvas : UserControl
    {
        private CreateLinkInteractor dataflowConnectionInteractor;
        private CreateLinkInteractor navflowConnectionInteractor;
        private WorkflowSelectionInteractor selectInteractor;
        private LibraryInteractor screenLibraryInteractor;
        private LibraryInteractor serviceLibraryInteractor;

        public event InteractorChangedHandler InteractorChangedEvent;
        public event InteractorChangedHandler InteractorResetEvent;

        private void OnInteractorReset(ViewInteractor newInteractor)
        {
            if(InteractorResetEvent != null)
            {
                InteractorResetEvent(this, newInteractor);
            }
        }

        private void OnInteractorChanged(ViewInteractor newInteractor)
        {
            if(InteractorChangedEvent != null)
            {
                InteractorChangedEvent(this, newInteractor);
            }
        }

        public WorkflowCanvas()
        {
            InitializeComponent();

            this.selectInteractor = new WorkflowSelectionInteractor(this.workflowDiagram1);
            this.screenLibraryInteractor = new LibraryInteractor(global::ProcessView.Properties.Resources.ScreenSymbolPNG);
            this.serviceLibraryInteractor = new LibraryInteractor(global::ProcessView.Properties.Resources.ServiceSymbolPNG);

            this.dataflowConnectionInteractor = new CreateLinkInteractor();
            this.dataflowConnectionInteractor.ObjectCreated += new GraphicObjectCreatedEventHandler(dataflowConnectionInteractor_ObjectCreated);
            this.dataflowConnectionInteractor.ValidateOperation += new EventHandler(dataflowConnectionInteractor_ValidateOperation);

            this.navflowConnectionInteractor = new CreateLinkInteractor();
            this.navflowConnectionInteractor.ObjectCreated += new GraphicObjectCreatedEventHandler(navflowConnectionInteractor_ObjectCreated);
            this.navflowConnectionInteractor.ValidateOperation += new EventHandler(navflowConnectionInteractor_ValidateOperation);
            
            this.screenLibraryInteractor.ValidateOperation += new EventHandler(libraryInteractor_ValidateOperation);
            this.serviceLibraryInteractor.ValidateOperation += new EventHandler(libraryInteractor_ValidateOperation);

            this.diagramView1.Interactor = this.selectInteractor;
            this.diagramView1.InteractorChanged += new EventHandler(diagramView1_InteractorChanged);
            this.diagramView1.KeyDown += new KeyEventHandler(diagramView1_KeyDown);
        }

        void diagramView1_KeyDown(object sender, KeyEventArgs e)
        {
            if(e.KeyCode == Keys.Escape)
            {
                this.DefaultToSelect();
            }
        }
        
        void libraryInteractor_ValidateOperation(object sender, EventArgs e)
	    {
		    this.DefaultToSelect();
        }

        void diagramView1_InteractorChanged(object sender, EventArgs e)
        {
            if((this.diagramView1.Interactor != null) && (this.diagramView1.Interactor is SelectInteractor) && (!(this.diagramView1.Interactor is WorkflowSelectionInteractor)))
            {
                DefaultToSelect();
            }
        }

        void dataflowConnectionInteractor_ValidateOperation(object sender, EventArgs e)
        {
            DefaultToSelect();
        }

        void dataflowConnectionInteractor_ObjectCreated(object sender, GraphicObjectCreatedEventArgs e)
        {
            Link link = e.GraphicObject as Link;
            link.ShapeType = LinkShapeType.Orthogonal;
            link.EndArrow = null;
            link.Stroke = new Stroke(Color.Red);
            //link.BevelBorder = true;
            //link.Stroke.Width = 6;
            link.CrossingStyle = LinkCrossingsStyle.Default;
            link.CanMove = true;
        }

        void navflowConnectionInteractor_ValidateOperation(object sender, EventArgs e)
        {
            DefaultToSelect();
        }

        void navflowConnectionInteractor_ObjectCreated(object sender, GraphicObjectCreatedEventArgs e)
        {
            Link link = e.GraphicObject as Link;
            link.ShapeType = LinkShapeType.Orthogonal;
            link.EndArrow = null;
            link.Stroke = new Stroke(Color.FromArgb(0,192,0));
            link.Stroke.Width = 7;
            link.BevelBorder = true;
            link.CrossingStyle = LinkCrossingsStyle.Default;
            link.CanMove = true;
        }

        public void DeleteSelection()
        {
            GraphicObject[] graphicObjects = this.diagramView1.Selection.GetSelectedObjects();

            foreach(GraphicObject go in graphicObjects)
            {
                go.RemoveFromParent();
            }
        }

        private void DefaultToSelect()
        {
            if((this.screenLibraryInteractor != null) && this.screenLibraryInteractor.InOperation)
            {
                this.screenLibraryInteractor.Stop();
            }

            if((this.serviceLibraryInteractor != null) && this.serviceLibraryInteractor.InOperation)
            {
                this.serviceLibraryInteractor.Stop();
            }

            this.diagramView1.Interactor = this.selectInteractor;

            OnInteractorChanged(this.selectInteractor);
            OnInteractorReset(this.selectInteractor);
        }
        
        public void AddServiceLibraryInteractor()
	    {
	        this.diagramView1.Interactor = this.serviceLibraryInteractor;
            this.serviceLibraryInteractor.Start(new ServiceWorkflowSymbol());
        }

        public void AddScreenLibraryInteractor()
        {
            this.diagramView1.Interactor = this.screenLibraryInteractor;
            this.screenLibraryInteractor.Start(new UserInterfaceWorkflowSymbol());
        }

        public void AddDataFlowConnection()
        {
            this.diagramView1.Interactor = this.dataflowConnectionInteractor;
        }

        public void AddNavFlowConnection()
        {
            this.diagramView1.Interactor = this.navflowConnectionInteractor;
        }

        public void AddService(string label)
        {
            IManageChildren container = this.diagramView1.Content as IManageChildren;
            if(container != null)
            {
                ServiceWorkflowSymbol serviceSymbol = new ServiceWorkflowSymbol();
                serviceSymbol.Text = label;
                serviceSymbol.Location = new Point2D(50f, 50f);
                
                container.AddChildren(new GraphicObject[] { serviceSymbol });
            }
        }

        public void AddUIScreen(string label)
        {
            IManageChildren container = this.diagramView1.Content as IManageChildren;
            if(container != null)
            {
                UserInterfaceWorkflowSymbol uiSymbol = new UserInterfaceWorkflowSymbol();
                uiSymbol.Text = label;
                uiSymbol.Location = new Point2D(50f, 50f);

                container.AddChildren(new GraphicObject[] { uiSymbol });
            }
        }

        public void ShowAll()
        {
            this.diagramView1.ShowAll();

        }

        public void IndicateErrors(bool checkState)
        {
            if (checkState == true)
            {
                this.workflowDiagram1.ShowErrors();
            }
            else
            {
                this.workflowDiagram1.HideErrors();
            }
        }

        /*
        public void StartServiceLibraryInteraction()
        {
            this.serviceLibraryInteractor.Start(new ServiceWorkflowSymbol());
        }

        public void StartScreenLibraryInteraction()
        {
            this.screenLibraryInteractor.Start(new UserInterfaceWorkflowSymbol());
        }
        */
    }
}
