using System;
using System.Collections.Generic;
using System.Collections;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Reflection;
using System.Text.RegularExpressions;

using ProcessScreenRepository;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Timers;

namespace SimulationEngine
{
    public class SimulationController
    {
        private DateTime _lastTimeMeasure = new DateTime();

        public delegate void ScreenScopeChangedHandler(object sender, ArrayList screens);
        public event ScreenScopeChangedHandler ScreenScopeChanged;

        public delegate void CurrentScreenChangedHandler(object sender, string newScreen);
        public event CurrentScreenChangedHandler CurrentScreenChanged;

        public delegate void SimulationStateChangedHandler(object sender, bool started);
        public event SimulationStateChangedHandler SimulationStateChanged;

        private bool _started = false;
        public bool Started
        {
            get
            {
                return _started;
            }
        }

        /// <summary>
        /// All the screens which are part of the simulated workflow
        /// </summary>
        private ArrayList _screens = new ArrayList();

        private const string WELCOMESCREEN = "screen0";
        private const string FIRSTSCREEN = "screen1";
        private const string LASTSCREEN = "screen99";

        /// <summary>
        /// The panel which is capable of showing a screen
        /// </summary>
        private SimulationPane _simuPane = null;

        private Window _serviceCallWindow = null;
        private System.Windows.Shapes.Rectangle _fadeOutRect = null;

        /// <summary>
        /// The screen which is currently displayed in _simuPane
        /// </summary>
        private string _currentScreen = null;
        public string CurrentScreen
        {
            get
            {
                return _currentScreen;
            }
        }

        public void SetCurrentScreen(string screenName)
        {
            if(screenName != null)
            {
                FrameworkElement screenPanel = this.GetScreenByName(screenName.ToLower());

                if(screenPanel != null)
                {
                    _currentScreen = screenName;
                    this._simuPane.SetScreen(screenPanel);
                    OnCurrentScreenChanged();
                }
            }
        }

        /*
        public void SwapScreenIn(string screenName)
        {
            if(screenName != null)
            {
                FrameworkElement screenPanel = this.GetScreenByName(screenName.ToLower());

                if(screenPanel != null)
                {
                    this._simuPane.SetScreen(screenPanel);
                }
            }
        }

        public void SwapScreenOut()
        {
            FrameworkElement screenPanel = this.GetScreenByName(WELCOMESCREEN);

            this._simuPane.SetScreen(screenPanel);
        }
        */

        private void OnCurrentScreenChanged()
        {
            if(CurrentScreenChanged != null)
            {
                CurrentScreenChanged(this, _currentScreen);
            }
        }

        private ArrayList _screenScope = new ArrayList();

        public void SelectScreen(string screenName)
        {
            ///
            /// MultiSelect
            ///

            IEnumerator it = _screenScope.GetEnumerator();

            string someScreen = null;
            bool screenExists = false;
            while(it.MoveNext())
            {
                someScreen = (string)(it.Current);

                if(someScreen.ToLower().Equals(screenName))
                {
                    screenExists = true;
                    break;
                }
            }

            if(screenExists)
            {
                //Remove from selection scope
                _screenScope.Remove(someScreen);
            }
            else
            {
                //Add to selection scope
                _screenScope.Add(screenName);
            }
        }

        /// <summary>
        /// Maintains the scope of the currently simulated workflow and
        /// shows the right screen upon user interaction with screens.
        /// </summary>
        /// <param name="canvas"></param>
        /// <param name="simulationPane"></param>
        public SimulationController(SimulationPane simulationPane)
        {
            if(simulationPane == null) throw new ArgumentNullException("SimulationController(CustomSymbols.DiagramCanvas canvas, SimulationPane simulationPane)");

            _simuPane = simulationPane;

            this.BuildScreens();

            
        }

        public void Dispose()
        {
            this._serviceCallWindow.Close();
        }

        /// <summary>
        /// Create all screens which are part of the workflow
        /// </summary>
        private void BuildScreens()
        {
            ArrayList buttons = new ArrayList();

            InitialScreen initialScreen = new InitialScreen();
            initialScreen.Name = WELCOMESCREEN;
            _screens.Add(initialScreen);

            ProcessScreen99 finalScreen = new ProcessScreen99();
            finalScreen.Name = LASTSCREEN;
            finalScreen.BtNewTest.Click += new RoutedEventHandler(BtNewTest_Click);
            _screens.Add(finalScreen);

            ProcessScreen1 screen1 = new ProcessScreen1();
            screen1.Name = FIRSTSCREEN;
            _screens.Add(screen1);
            buttons.AddRange(screen1.GetButtons());
            screen1.LoginSubmittedEvent += new ProcessScreen1.LoginSubmittedEventHandler(screen1_LoginSubmittedEvent);

            ProcessScreen2 screen2 = new ProcessScreen2();
            screen2.Name = "screen2";
            _screens.Add(screen2);
            buttons.AddRange(screen2.GetButtons());

            ProcessScreen3 screen3 = new ProcessScreen3();
            screen3.Name = "screen3";
            _screens.Add(screen3);
            buttons.AddRange(screen3.GetButtons());

            ProcessScreen4 screen4 = new ProcessScreen4();
            screen4.Name = "screen4";
            _screens.Add(screen4);
            buttons.AddRange(screen4.GetButtons());

            ProcessScreen5 screen5 = new ProcessScreen5();
            screen5.Name = "screen5";
            _screens.Add(screen5);
            buttons.AddRange(screen5.GetButtons());

            ProcessScreen6 screen6 = new ProcessScreen6();
            screen6.Name = "screen6";
            _screens.Add(screen6);
            buttons.AddRange(screen6.GetButtons());

            ServiceScreen1 service1 = new ServiceScreen1();
            service1.Name = "service1";
            _screens.Add(service1);
            buttons.AddRange(service1.GetButtons());

            ServiceScreen2 service2 = new ServiceScreen2();
            service2.Name = "service2";
            _screens.Add(service2);
            buttons.AddRange(service2.GetButtons());

            ServiceScreen3 service3 = new ServiceScreen3();
            service3.Name = "service3";
            _screens.Add(service3);
            buttons.AddRange(service3.GetButtons());

            ServiceScreen4 service4 = new ServiceScreen4();
            service4.Name = "service4";
            _screens.Add(service4);
            buttons.AddRange(service4.GetButtons());

            ServiceScreen5 service5 = new ServiceScreen5();
            service5.Name = "service5";
            _screens.Add(service5);
            buttons.AddRange(service5.GetButtons());

            ServiceScreen6 service6 = new ServiceScreen6();
            service6.Name = "service6";
            _screens.Add(service6);
            buttons.AddRange(service6.GetButtons());

            foreach(System.Windows.Controls.Button button in buttons)
            {
                button.Click += new RoutedEventHandler(button_Click);
            }

            // Create Service Activation Window
            ActivateServiceCall serviceCallDialog = new ActivateServiceCall();
            //Layout within parent
            serviceCallDialog.VerticalAlignment = VerticalAlignment.Center;
            serviceCallDialog.HorizontalAlignment = HorizontalAlignment.Center;

            _serviceCallWindow = new Window();
            _serviceCallWindow.WindowStyle = WindowStyle.None;
            _serviceCallWindow.WindowStartupLocation = WindowStartupLocation.CenterOwner;
            _serviceCallWindow.BorderThickness = new Thickness(0.0d);
            _serviceCallWindow.BorderBrush = null;
            _serviceCallWindow.Width = serviceCallDialog.Width;
            _serviceCallWindow.Height = serviceCallDialog.Height;

            Grid rootLayout = new Grid();

            rootLayout.VerticalAlignment = VerticalAlignment.Stretch;
            rootLayout.HorizontalAlignment = HorizontalAlignment.Stretch;
            rootLayout.Children.Add(serviceCallDialog);

            _serviceCallWindow.Content = rootLayout;

            // Fade out rectangle
            //Brush fadeColor = new SolidColorBrush(Color.FromArgb(128, 40, 40, 40));
            //_fadeOutRect = new System.Windows.Shapes.Rectangle();
            //_fadeOutRect.Fill = fadeColor;
        }

        void screen1_LoginSubmittedEvent(ProcessScreen1 sender, LoginEventArgs e)
        {
            FrameworkElement fe = this.GetScreenByName("service1");
            if(fe is ServiceScreen1)
            {
                //((ServiceScreen1)fe).
            }
        }

        /// <summary>
        /// Callback method to reset the test when requested on the final test report
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void BtNewTest_Click(object sender, RoutedEventArgs e)
        {
            this.Start();
        }

        /// <summary>
        /// Mouse Click Handler which processes
        /// all interactions with navigational elements
        /// in the simulated screens
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void button_Click(object sender, RoutedEventArgs e)
        {
            // Some button has been clicked: show the new screen...

            LogInteraction(this.CurrentScreen);

            Button button = (Button)(e.Source);

            string panelName = this.ParsePanelName(button.Name);

            SetCurrentScreen(panelName);
        }

        private void LogInteraction(string panelName)
        {
            DateTime currentTimeMeasure = DateTime.Now;

            TimeSpan timeDiff = _lastTimeMeasure - currentTimeMeasure;

            string logEntry = string.Format("Screen '{0}' took {1} seconds to complete.", panelName, timeDiff.TotalSeconds);

            //TODO: Write the logEntry to a log collection for later evaluation.

            _lastTimeMeasure = currentTimeMeasure;
        }

        /// <summary>
        /// Helper method to extract a panel name from a button name.
        /// </summary>
        /// <param name="buttonName"></param>
        /// <returns></returns>
        private string ParsePanelName(string buttonName)
        {
            string panelName = null;

            Regex regex = new Regex(@"^bt\w*(?<panelname>(screen|service)\d\d?)$", RegexOptions.IgnoreCase);
            Match match = regex.Match(buttonName);
            if(match.Success && (match.Groups.Count > 0))
            {
                panelName = match.Groups["panelname"].Value;
            }

            return panelName;
        }

        /// <summary>
        /// Looksup the screen with the provided name
        /// from the collection of workflow screens
        /// </summary>
        /// <param name="name"></param>
        /// <returns></returns>
        private FrameworkElement GetScreenByName(string name)
        {
            FrameworkElement element = null;

            // From the list of instantiated screens, lookup the specified screen
            foreach(FrameworkElement e in this._screens)
            {
                if((e.Name != null) && e.Name.ToLower().Equals(name))
                {
                    element = e;
                    break;
                }
            }

            return element;
        }

        /// <summary>
        /// Starts the simulation process
        /// </summary>
        public void Start()
        {
            if(this.Started)
            {
                // Stop Simulaiton
                _started = false;

                ClearSelection();

                this.SetCurrentScreen(WELCOMESCREEN);
            }
            else
            {
                // Start Simulation
                _started = true;

                //this.ShowStartDialog();

                this._lastTimeMeasure = DateTime.Now;

                this.SetCurrentScreen(FIRSTSCREEN);
            }

            // Notify the Canvas to reset the filling
            OnSimulationStateChanged(_started);
        }

        private void OnSimulationStateChanged(bool state)
        {
            if(SimulationStateChanged != null)
            {
                SimulationStateChanged(this, state);
            }
        }

        public void ClearSelection()
        {
            this._screenScope.Clear();
        }

        public void ShowStartDialog()
        {
            _serviceCallWindow.Owner = Application.Current.MainWindow;
            _serviceCallWindow.ShowDialog();
        }

        private void HideStartDialog()
        {
            _serviceCallWindow.Hide();
        }
    }

    /*
    public enum SelectionScope
    {
        All,
        MultiSelect
    }
     * */
}
